package init_serve

import (
	"eta_gn/eta_bridge/global"
	"eta_gn/eta_bridge/services/binlog"
	"fmt"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
	"io"
	"log"
	"os"
	"time"
)

func Mysql() {
	mysqlConf := global.CONFIG.Mysql
	if len(mysqlConf.List) <= 0 {
		global.LOG.Error("mysql链接未配置")
		panic(fmt.Errorf("mysql链接未配置"))
	}

	logWriter := io.MultiWriter(global.MYSQL_LOG) //binlog日志，记录到文件中去
	if global.CONFIG.Mysql.Stdout {
		logWriter = io.MultiWriter(global.MYSQL_LOG, os.Stdout)
	}
	newLogger := logger.New(log.New(logWriter, "\r\n", log.LstdFlags), logger.Config{
		SlowThreshold:             200 * time.Millisecond, //慢sql ：200ms
		LogLevel:                  logger.Info,            //记录的日志类型，info代表所有信息都记录
		IgnoreRecordNotFoundError: true,                   //是否忽略找不到数据错误信息（只是日志记录记录成err还是普通的输出的区别，并不影响业务代码中的：找不到数据行error）
		Colorful:                  true,                   //是否颜色输出
	})

	mysqlMap := make(map[string]*gorm.DB)
	for _, conf := range mysqlConf.List {
		dsn := conf.Dsn

		db, err := gorm.Open(mysql.Open(dsn), &gorm.Config{
			Logger: newLogger,
		})
		if err != nil {
			global.LOG.Errorf("mysql 启动异常，数据库:", conf.AliasName, ";Err：", err)
			panic(fmt.Errorf("mysql 启动异常，数据库:", conf.AliasName, "Err:%s", err))
		}

		sqlDB, err := db.DB()
		if err != nil {
			global.LOG.Errorf("mysql 创建连接池失败，数据库:", conf.AliasName, ";Err：", err)
			panic(fmt.Errorf("mysql 创建连接池失败，数据库:", conf.AliasName, "Err:%s", err))
		}

		mysqlMap[conf.AliasName] = db

		if mysqlConf.DefaultDsnAliasName == conf.AliasName {
			global.DEFAULT_MYSQL = db
		}

		sqlDB.SetMaxIdleConns(conf.MaxIdleConns)

		sqlDB.SetMaxOpenConns(conf.MaxOpenConns)

	}

	global.MYSQL = mysqlMap

	if mysqlConf.Binlog.IsListen {
		go binlog.ListenMysql()
	}

	fmt.Println("mysql init end")
}
